@@ -12,7 +12,8 @@ from pyperf._cpu_utils import (format_cpu_list, parse_cpu_list,
                                set_highest_priority)
 from pyperf._formatter import format_timedelta
 from pyperf._utils import (MS_WINDOWS, MAC_OS, abs_executable,
-                           WritePipe, get_python_names)
+                           WritePipe, get_python_names,
+                           merge_profile_stats)
 from pyperf._worker import WorkerProcessTask
 
 
@@ -48,6 +49,19 @@ def parse_python_names(names):
     return parts
 
 
+def profiling_wrapper(func):
+    """
+    Wrap a function to collect profiling.
+    """
+    import cProfile
+    profiler = cProfile.Profile()
+
+    def profiling_func(*args):
+        return profiler.runcall(func, *args)
+
+    return profiler, profiling_func
+
+
 class CLIError(Exception):
     pass
 
@@ -63,6 +77,13 @@ class Runner:
                  program_args=None, add_cmdline_args=None,
                  _argparser=None):
 
+        # Reset the stats collection if running a --enable-pystats build
+        try:
+            sys._stats_off()
+            sys._stats_clear()
+        except AttributeError:
+            pass
+
         # Watchdog: ensure that only once instance of Runner (or a Runner
         # subclass) is created per process to prevent bad suprises
         cls = self.__class__
@@ -222,6 +243,11 @@ class Runner:
                                  'PYTHON as CHANGED_NAME '
                                  'and REF_PYTHON as REF_NAME in results')
 
+        parser.add_argument('--profile',
+                            type=str,
+                            help='Collect profile data using cProfile '
+                                 'and output to the given file.')
+
         memory = parser.add_mutually_exclusive_group()
         memory.add_argument('--tracemalloc', action="store_true",
                             help='Trace memory allocations using tracemalloc')
@@ -305,6 +331,8 @@ class Runner:
             self._only_in_worker("--worker-task")
 
         if args.tracemalloc:
+            if args.action == 'command':
+                raise CLIError('--tracemalloc cannot be used with pyperf command')
             try:
                 import tracemalloc   # noqa
             except ImportError as exc:
@@ -454,12 +482,21 @@ class Runner:
         if not self._check_worker_task():
             return None
 
+        if self.args.profile:
+            profiler, time_func = profiling_wrapper(time_func)
+
         def task_func(task, loops):
             return time_func(loops, *args)
 
         task = WorkerProcessTask(self, name, task_func, metadata)
+
         task.inner_loops = inner_loops
-        return self._main(task)
+        result = self._main(task)
+
+        if self.args.profile:
+            merge_profile_stats(profiler, self.args.profile)
+
+        return result
 
     def bench_func(self, name, func, *args, **kwargs):
         """"Benchmark func(*args)."""
@@ -474,6 +511,9 @@ class Runner:
         if args:
             func = functools.partial(func, *args)
 
+        if self.args.profile:
+            profiler, func = profiling_wrapper(func)
+
         def task_func(task, loops):
             # use fast local variables
             local_timer = time.perf_counter
@@ -494,7 +534,12 @@ class Runner:
 
         task = WorkerProcessTask(self, name, task_func, metadata)
         task.inner_loops = inner_loops
-        return self._main(task)
+        result = self._main(task)
+
+        if self.args.profile:
+            merge_profile_stats(profiler, self.args.profile)
+
+        return result
 
     def bench_async_func(self, name, func, *args, **kwargs):
         """Benchmark await func(*args)"""
@@ -509,6 +554,9 @@ class Runner:
         if args:
             func = functools.partial(func, *args)
 
+        if self.args.profile:
+            profiler, func = profiling_wrapper(func)
+
         def task_func(task, loops):
             if loops != 1:
                 async def main():
@@ -549,7 +597,12 @@ class Runner:
 
         task = WorkerProcessTask(self, name, task_func, metadata)
         task.inner_loops = inner_loops
-        return self._main(task)
+        result = self._main(task)
+
+        if self.args.profile:
+            merge_profile_stats(profiler, self.args.profile)
+
+        return result
 
     def timeit(self, name, stmt=None, setup="pass", teardown="pass",
                inner_loops=None, duplicate=None, metadata=None, globals=None):
@@ -671,6 +724,9 @@ class Runner:
         if not self._check_worker_task():
             return None
 
+        if self.args.profile:
+            command.extend(["--profile", self.args.profile])
+
         # Use lazy import to limit imports on 'import pyperf'
         from pyperf._command import BenchCommandTask
         task = BenchCommandTask(self, name, command)
